package com.adc.da.vdaims.aspect;

import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.adc.da.mp.base.IBaseService;
import com.adc.da.util.utils.CollectionUtils;
import com.adc.da.util.utils.StringUtils;
import com.adc.da.vdaims.annotation.TableAuthCheck;
import com.adc.da.vdaims.enums.ParamType;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Slf4j
@Component
@Aspect
public class TableAuthCheckAspect {

    @Autowired
    private ApplicationContext applicationContext;

    /**
     * 配置切入点
     */
    @Pointcut("@annotation(com.adc.da.vdaims.annotation.TableAuthCheck)")
    public void pointcut() {

    }

    @Around("pointcut()")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        MethodSignature signature = (MethodSignature) point.getSignature();
        Method method = signature.getMethod();
        TableAuthCheck tableAuthCheck = method.getAnnotation(TableAuthCheck.class);
        if (tableAuthCheck != null) {
            // 如果存在.， 则需要从bean里面取
            String[] paramNameArr = tableAuthCheck.paramName().split("\\.");

            int index = -1;
            String[] parameterNames = signature.getParameterNames();
            for (int i = 0; i < parameterNames.length; i++) {
                if (parameterNames[i].equals(paramNameArr[0])) {
                    index = i;
                    break;
                }
            }
            if (index == -1) {
                throw new RuntimeException(tableAuthCheck.paramName() + "NOT FOUND!");
            }
            List<String> paramValue;
            if (paramNameArr.length > 1) {
                JSON json = JSONUtil.parse(point.getArgs()[index]);
                paramValue = tableAuthCheck.paramType() == ParamType.string ? new ArrayList(){{add(json.getByPath(paramNameArr[1], String.class));}} : json.getByPath(paramNameArr[1], List.class);
            } else {
                Object arg = point.getArgs()[index];
                paramValue = tableAuthCheck.paramType() == ParamType.string ? new ArrayList(){{add(arg);}} : (List<String>) arg;
            }

            HttpServletRequest request = ((ServletRequestAttributes) Objects.requireNonNull(RequestContextHolder.getRequestAttributes())).getRequest();
            Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
            String qyId = CollectionUtils.isNotEmpty(usermap) ? usermap.get("USERNAME").toString() : StringUtils.EMPTY;

            IBaseService baseService = applicationContext.getBean(tableAuthCheck.service());
            int count = baseService.count(new QueryWrapper<>().eq(tableAuthCheck.qyId(), qyId).in(tableAuthCheck.queryColunm(), paramValue));
            if (count == 0) {
                StringBuilder msg = new StringBuilder();
                msg.append("表权限校验不通过").append(tableAuthCheck.service().getName()).append(",query={").append(tableAuthCheck.qyId()).append("=").append(qyId).append(",")
                        .append(tableAuthCheck.queryColunm()).append("=").append(paramValue).append("}");
                throw new RuntimeException(msg.toString());
            }

        }
        return point.proceed();
    }
}
